%{ open Ast %}

%token SEMI OPNPR CLSEPR OPNBR CLSEBR COMMA
%token PLUS MINUS TIMES DIVIDE ASSIGN
%token EQ NEQ LT LEQ GT GEQ
%token RETURN IF ELSE FOR WHILE INT NOTE RHYTHM MOTIF
%token PLUS MINUS STAR SLASH ASSIGN SHARP FLAT
%token HALF QUARTER EIGHTH SIXTEENTH
%token DHALF DQUARTER DEIGHTH
%token GET TRANSPOSE JOT
%token <int> LITERAL BASE
%token <string> ID
%token EOF

%nonassoc NOELSE
%nonassoc ELSE
%nonassoc SHARPD
%nonassoc FLATD
  
%left ASSIGN
%left EQ NEQ
%left LT GT LEQ GEQ
%left PLUS MINUS
%left TIMES DIVIDE

%start program
%type <Ast.program> program

%%

program:
   /* nothing */           { [], [] }
 | program vdecl           { ($2 :: fst $1), snd $1 }
 | program fdecl           { fst $1, ($2 :: snd $1) }

fdecl:
   ID OPNPR formals_opt CLSEPR OPNBR vdecl_list stmt_list CLSEBR
                           { { fname = $1;
	                           formals = $3;
	                           locals = List.rev $6;
	                           body = List.rev $7 } }

formals_opt:
    /* nothing */           { [] }
  | formal_list             { List.rev $1 }

formal_list:
    ID                      { [$1] }
  | formal_list COMMA ID    { $3 :: $1 }

vdecl_list:
    /* nothing */           { [] }
  | vdecl_list vdecl        { $2 :: $1 }

vdecl:
    INT ID SEMI             { $2 }
  | MOTIF ID SEMI           { $2 }
  | NOTE ID SEMI            { $2 }
  | RHYTHM ID SEMI          { $2 }

stmt_list:
    /* nothing */           { [] }
  | stmt_list stmt          { $2 :: $1 }

stmt:
    expr SEMI               { Expr($1) }
  | RETURN expr SEMI        { Return($2) }
  | OPNBR stmt_list CLSEBR  { Block(List.rev $2) }
  | IF OPNPR expr CLSEPR stmt %prec NOELSE 
                            { If($3, $5, Block([])) }
  | IF OPNPR expr CLSEPR stmt ELSE stmt    
                            { If($3, $5, $7) }
  | FOR OPNPR expr_opt SEMI expr_opt SEMI expr_opt CLSEPR stmt
                            { For($3, $5, $7, $9) }
  | WHILE OPNPR expr CLSEPR stmt 
                            { While($3, $5) }
  
expr_opt:
    /* nothing */           { Noexpr }
  | expr                    { $1 }

note:
    BASE                    { $1 }
  | BASE FLAT  %prec FLATD  { $1 - 1 }
  | BASE SHARP %prec SHARPD { $1 + 1 }

rhythm:
    SIXTEENTH               { 1 }
  | EIGHTH                  { 2 }
  | DEIGHTH                 { 3 }
  | QUARTER                 { 4 }
  | DQUARTER                { 6 }
  | HALF                    { 8 }
  | DHALF                   { 12 }
  
expr:
    LITERAL                 { Literal($1) }
  | ID                      { Id($1) }
  | rhythm                  { Note($1) }
  | note                    { Rhythm($1) }
  | expr PLUS   expr        { Binop($1, Add,      $3) }
  | expr MINUS  expr        { Binop($1, Subtract, $3) }
  | expr TIMES  expr        { Binop($1, Multipy,  $3) }
  | expr DIVIDE expr        { Binop($1, Divide,   $3) }
  | expr EQ     expr        { Binop($1, Equal,    $3) }
  | expr NEQ    expr        { Binop($1, Neq,      $3) }
  | expr LT     expr        { Binop($1, Less,     $3) }
  | expr LEQ    expr        { Binop($1, Leq,      $3) }
  | expr GT     expr        { Binop($1, Greater,  $3) }
  | expr GEQ    expr        { Binop($1, Geq,   $3) }
  | ID ASSIGN expr          { Assign($1, $3) }
  | ID OPNPR actuals_opt CLSEPR { Call($1, $3) }
  | OPNPR expr CLSEPR       { $2 }
  
note_set:
   note COMMA rhythm       { $1, $3 }
  
actuals_opt:
    /* nothing */           { [] }
  | actuals_list            { List.rev $1 }

actuals_list:
    expr                    { [$1] }
  | actuals_list COMMA expr { $3 :: $1 }