{ open Parser }

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "/*"     { comment lexbuf }           (* Comments *)
| '('      { OPNPR }
| ')'      { CLSEPR }
| '{'      { OPNBR }
| '}'      { CLSEBR }
| ';'      { SEMI }
| ','      { COMMA }
| '+'      { PLUS }
| '-'      { MINUS }
| '*'      { TIMES }
| '/'      { DIVIDE }
| '='      { ASSIGN }
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| '#'      { SHARP }
| 'b'      { FLAT }
| "SX"     { SIXTEENTH }
| "EG"     { EIGHTH }
| "QT"     { QUARTER }
| "HF"     { HALF }
| "DE"     { DEIGHTH }
| "DQ"     { DQUARTER }
| "DH"     { DHALF }
| "get"    { GET }
| "if"     { IF }
| "note"   { NOTE }
| "rhythm" { RHYTHM }
| "else"   { ELSE }
| "for"    { FOR }
| "while"  { WHILE }
| "return" { RETURN }
| "motif"  { MOTIF } 
| "int"    { INT }
| "transpose"               { TRANSPOSE }
| "jot"                     { JOT }
| ['0'-'9']+ as lxm { LITERAL(int_of_string lxm) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| ['0'-'9']['A'-'G'] as entr                                (* suedo-base12 *)
                            { BASE(((int_of_char entr.[0] - 48) * 12 ) +
                                ((int_of_char entr.[1] - 64) * 16 / 10)) }         
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "*/" { token lexbuf }
| _    { comment lexbuf }